import { ApiProperty, IntersectionType, PartialType } from '@nestjs/swagger';
import { IsArray, IsNumber, IsOptional, IsString } from 'class-validator';
import { PagerDto } from '~/common/dto/pager.dto';
import { Storage } from '../tools/storage/storage.entity';

export class ProductDto {
  @ApiProperty({ description: '产品名称' })
  @IsString()
  name: string;

  @ApiProperty({ description: '单位(字典)' })
  @IsOptional()
  @IsNumber()
  unitId: number;

  @ApiProperty({ description: '所属公司' })
  @IsOptional()
  @IsNumber()
  companyId: number;

  @ApiProperty({ description: '附件' })
  files: Storage[];
}

export class ProductUpdateDto extends PartialType(ProductDto) {
  @ApiProperty({ description: '附件' })
  @IsOptional()
  @IsArray()
  fileIds: number[];
}

export class ProductQueryDto extends IntersectionType(
  PagerDto<ProductDto>,
  PartialType(ProductDto)
) {
  @ApiProperty({ description: '所属公司名称' })
  @IsOptional()
  @IsString()
  company: string;

  @ApiProperty({ description: '产品名字' })
  @IsOptional()
  @IsString()
  name?: string;
}
